package me.icymint.libra.sage.core.query;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

import me.icymint.libra.sage.core.param.DefaultParamHandler;
import me.icymint.libra.sage.core.param.ParamHandler;
import me.icymint.libra.sage.core.result.ResultHandler;
import me.icymint.libra.sage.model.SqlVarType;

/**
 * 
 * 基础查询。该抽象类为JDBC执行的模板代码，完成通过一个给定的Connection获取查询结果的步骤。 该步骤由接口{@link SqlQuery}规定。
 * 
 * @author Daniel
 * @datetime 2012-12-20上午10:44:29
 * 
 * @param <T>
 *            查询结果
 * @param <U>
 *            预处理语句类型
 * @param <S>
 *            提供SQL语句的对象
 * @param <V>
 *            返回结果类型，可以为ResultSet或者Integer，分别表示查询结果和执行结果码。
 */
public abstract class AbstractSqlQuery<T, U extends PreparedStatement, S, V>
		implements SqlQuery<T> {

	/**
	 * 提供SQL执行语句的对象。
	 */
	private S sql;
	/**
	 * 本次查询获取结果的最大行数。
	 */
	private int maxRows;
	/**
	 * 结果处理对象，把{@link V}对象转换成结果{@link T}
	 */
	private ResultHandler<T> rh;
	/**
	 * 参数处理对象，把参数列表输入到预处理语句中。
	 */
	private ParamHandler<U> ph = new DefaultParamHandler<U>();
	/**
	 * 每个参数对应的Jdbc类型，参考{@link Types}和{@link SqlVarType}。
	 */
	private int[] types;

	protected AbstractSqlQuery(S sql, ResultHandler<T> rh, int maxRows,
			int[] types) {
		this.sql = sql;
		this.rh = rh;
		if (maxRows < 0)
			maxRows = 0;
		this.maxRows = maxRows;
		this.types = types;
	}

	/**
	 * 
	 * @return 获取最大行数。
	 */
	protected int getMaxRows() {
		return maxRows;
	}

	/**
	 * 获取不同的Statement语句。
	 * 
	 * @param conn
	 * @param sql
	 * @throws SQLException
	 */
	protected abstract U getStatement(Connection conn, String sql)
			throws SQLException;

	/**
	 * 
	 * @return 获取参数列表对应类型。
	 */
	protected int[] getTypes() {
		return types;
	}

	/**
	 * 
	 * @param rh
	 * @param v
	 * @return 处理Jdbc查询结果，得到正确的类型。
	 * @throws SQLException
	 */
	protected abstract T handle(ResultHandler<T> rh, V v) throws SQLException;

	/**
	 * 该步骤在本类中被进一步分解，实现更细粒度的操作和接口。
	 */
	@Override
	public T query(Connection conn) throws SQLException {
		String sql = sql(this.sql);
		U pps = this.getStatement(conn, sql);
		if (getMaxRows() > 0)
			pps.setMaxRows(getMaxRows());
		T t;
		try {
			t = handle(rh, query(pps, ph));
		} catch (SQLException e) {
			throw new SQLException("执行语句：" + sql, e);
		} finally {
			pps.close();
		}
		return t;
	}

	/**
	 * 执行预处理语句，获取查询结果。
	 * 
	 * @param pps
	 * @param ph
	 * @return 查询结果。
	 * @throws SQLException
	 */
	protected abstract V query(U pps, ParamHandler<U> ph) throws SQLException;

	/**
	 * 返回可以获取的SQL对象。
	 * 
	 * @param sql
	 * @return
	 * @throws SQLException
	 */
	protected abstract String sql(S sql) throws SQLException;
}
