package me.icymint.libra.sage.model;

import java.sql.SQLException;
import java.util.List;

import me.icymint.libra.sage.core.JdbcUtils;
import me.icymint.libra.sage.model.format.DefaultFormat;
import me.icymint.libra.sage.model.format.FormatSql;
import me.icymint.libra.sage.model.format.SqlFormatException;
import me.icymint.libra.sage.model.object.FunctionManagerObject;
import me.icymint.libra.sage.model.object.SqlObjectType;
import me.icymint.libra.sage.model.operator.ManagerObjects;
import me.icymint.libra.sage.model.operator.SqlManager;
import me.icymint.libra.sage.model.orm.Mappings;
import me.icymint.libra.sage.model.token.ProcedureToken;

/**
 * SQL存储过程对象。
 * <p>
 * 其集成了执行存储过程的命令。 共有三种方式可以调用。
 * <p>
 * 1. 无返回值调用<br/>
 * {@link #commit(Object...)};<br/>
 * <br/>
 * 2. 普通返回值，返回数组结果。<br/>
 * {@link #call(Object...)};<br/>
 * <br/>
 * 3. 返回组装好的对象。<br/>
 * {@link #load(Class, Object...)};<br/>
 * 该种方式需要按照{@link Mappings}类的规则进行字段映射。
 * 
 * @see Mappings
 * @author Daniel
 * @datetime 2013-1-15上午11:59:31
 * 
 */
public class SqlProcedure extends FunctionManagerObject implements SqlManager {
	private final ManagerObjects<SqlVar> mo;
	private final String sql;
	private final FormatSql format;

	protected SqlProcedure(SqlInfo si, String prefix, String name,
			boolean isMask, SqlVar[] sv, SqlTable[] st, SqlFunction[] sf,
			String sql, SqlVar... out) {
		super(si, prefix, name, isMask, sv, st, sf);
		mo = new ManagerObjects<SqlVar>(out, SqlVar.class);
		this.sql = sql;
		format = new DefaultFormat(getJdbcUtils().getDialect(), this);
	}

	/**
	 * 用于具有返回值的存储过程。
	 * 
	 * @param param
	 * @throws SQLException
	 */
	public <T> T call(Object... param) throws SQLException {
		return this.getJdbcUtils().query(execSql(),
				JdbcUtils.getTypes(getVars()), param);
	}

	/**
	 * 执行存储过程，返回一行记录。
	 * 
	 * @param sql
	 * @param param
	 * @throws SQLException
	 */
	public Object[] callArray(Object... param) throws SQLException {
		return this.getJdbcUtils().queryArray(execSql(),
				JdbcUtils.getTypes(getVars()), param);
	}

	/**
	 * 执行存储过程，返回多行记录
	 * 
	 * @param sql
	 * @param offset
	 * @param maxRows
	 * @param param
	 * @throws SQLException
	 */
	public List<Object[]> callArrayList(int offset, int maxRows,
			Object... param) throws SQLException {
		return this.getJdbcUtils().queryArrayList(execSql(), offset, maxRows,
				JdbcUtils.getTypes(getVars()), param);
	}

	/**
	 * 执行存储过程，返回一列值。
	 * 
	 * @param sql
	 * @param offset
	 * @param maxRows
	 * @param param
	 * @throws SQLException
	 */
	public <T> List<T> callList(int offset, int maxRows, Object... param)
			throws SQLException {
		return this.getJdbcUtils().queryList(execSql(), offset, maxRows,
				JdbcUtils.getTypes(getVars()), param);
	}

	/**
	 * 检查参数匹配。
	 * 
	 * @param param
	 */
	public boolean checkParam(Object... param) {
		return param.length == this.getVars().length;
	}

	/**
	 * 用于执行没有返回值的存储过程。
	 * 
	 * @param param
	 * @throws SQLException
	 */
	public void commit(Object... param) throws SQLException {
		new ProcedureToken(this).addTokens(param).commit();
	}

	/**
	 * 用于执行没有返回值的存储过程。
	 * 
	 * @param params
	 * @throws SQLException
	 */
	public <V> void commitAll(V[]... params) throws SQLException {
		ProcedureToken pt = new ProcedureToken(this);
		for (V[] param : params) {
			pt.addTokens(param);
		}
		pt.commit();
	}

	@Override
	public void create() throws SQLException {
		this.getJdbcUtils().execute(
				this.getJdbcUtils().getDialect().getCreateProcedureSql(this));
	}

	@Override
	public void createIfNotExist() {
		try {
			this.create();
		} catch (SQLException e) {
		}
	}

	@Override
	public void delete() throws SQLException {
		this.getJdbcUtils().execute(
				this.getJdbcUtils().getDialect().getDeleteProcedureSql(this));
	}

	/**
	 * 获取执行语句。
	 */
	public String execSql() {
		return this.getJdbcUtils().getDialect().executeProc(this);
	}

	/**
	 * 获取格式化完毕的SQL，用于查看语句。
	 * 
	 * @throws SqlFormatException
	 */
	public String getFormatSql() throws SqlFormatException {
		return format.format(sql);
	}

	/**
	 * 返回值，要使{@link #load(Class, Object...)}有效，必须返回正确的返回字段列表。
	 */
	public SqlVar[] getOutVars() {
		return mo.getAll();
	}

	/**
	 * 存储过程核心代码。扩展以后需要有对高级查询的支持。
	 */
	public String getSql() {
		return sql;
	}

	@Override
	protected SqlObjectType getSqlObjectType() {
		return SqlObjectType.Procedure;
	}

	/**
	 * 直接从call中获取第一个完整对象。
	 * 
	 * @param cls
	 * @param param
	 * @throws SQLException
	 */
	public <T> T load(Class<T> cls, Object... param) throws SQLException {
		return loads(cls, 1, 1, param).get(0);
	}

	/**
	 * 直接从call中获取完整对象列表。
	 * 
	 * @param cls
	 * @param offset
	 * @param maxRows
	 * @param param
	 * @throws SQLException
	 */
	public <T> List<T> loads(Class<T> cls, int offset, int maxRows,
			Object... param) throws SQLException {
		if (getOutVars().length == 0)
			throw new SQLException("该存储过程" + getName() + "不能以此种方式执行");
		return this.getJdbcUtils().loadList(execSql(), cls, getOutVars(),
				offset, maxRows, getVars(), param);
	}

	public void setOutVars(SqlVar[] vs) {
		mo.clear();
		mo.addAll(vs);
	}
}
