package me.icymint.libra.sage.model.format;

import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import me.icymint.libra.sage.core.dialect.SqlDialect;
import me.icymint.libra.sage.model.SqlFunction;
import me.icymint.libra.sage.model.SqlProcedure;
import me.icymint.libra.sage.model.SqlTable;
import me.icymint.libra.sage.model.SqlVar;
import me.icymint.libra.sage.model.operator.FunctionManager;
import me.icymint.libra.sage.model.operator.SQLNotExistsException;
import me.icymint.libra.sage.model.operator.TableManager;
import me.icymint.libra.sage.model.operator.VarManager;

/**
 * 完成对Sql对象{@link SqlFunction}、{@link SqlProcedure}等创建语句的格式化转换。
 * 转换的形式是把$t(0),$t(0,1),$v(0),$f(0)或者$t(table),$t(table,var),$v(var),$f(func)一类
 * Sage自定义的格式码转换成Sql数据库可识别的Sql语句。
 * 
 * @author Daniel
 * @datetime 2012-11-14上午9:23:41
 * 
 */
public class DefaultFormat extends AbstractFormat {

	private static PatternFormat[] get(Object obj, SqlDialect sd) {
		List<PatternFormat> pf = new LinkedList<PatternFormat>();
		if (obj instanceof VarManager) {
			pf.add(getVarFormat((VarManager) obj, sd));
		}
		if (obj instanceof TableManager) {
			pf.add(getTableFormat((TableManager) obj));
			pf.add(getTableVarFormat((TableManager) obj));
		}
		if (obj instanceof FunctionManager) {
			pf.add(getFunctionFormat((FunctionManager) obj));
		}
		return pf.toArray(new PatternFormat[] {});

	}

	public static PatternFormat getFunctionFormat(final FunctionManager fm) {
		return new PatternFormat(Pattern.compile("\\$f\\(" + NAME_ID + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {
				SqlFunction v = null;
				try {
					try {
						v = fm.getFunction(Integer.valueOf(m.group(1)));
					} catch (NumberFormatException e) {
						v = fm.getFunction(m.group(1));
					}
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"函数"), e);
				}
				return v.toString();
			}

		};
	}

	public static PatternFormat getTableFormat(final TableManager tm) {
		return new PatternFormat(Pattern.compile("\\$t\\(" + NAME_ID + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {
				SqlTable v = null;
				try {
					try {
						v = tm.getTable(Integer.valueOf(m.group(1)));
					} catch (NumberFormatException e) {
						v = tm.getTable(m.group(1));
					}
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"表格"), e);
				}
				return v.toString();
			}

		};
	}

	public static PatternFormat getTableVarFormat(final TableManager tm) {
		return new PatternFormat(Pattern.compile("\\$t\\(" + NAME_ID + ","
				+ NAME_ID + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {

				SqlTable t = null;
				SqlVar v = null;
				try {
					try {
						t = tm.getTable(Integer.valueOf(m.group(1)));
					} catch (NumberFormatException e) {
						t = tm.getTable(m.group(1));
					}
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"表格"), e);
				}

				try {
					try {
						v = t.getVar(Integer.valueOf(m.group(2)));
					} catch (NumberFormatException e) {
						v = t.getVar(m.group(2));
					}
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"字段"), e);
				}

				return t.toString() + "." + v.toString();
			}

		};
	}

	public static PatternFormat getVarFormat(final VarManager vm,
			final SqlDialect sd) {
		return new PatternFormat(Pattern.compile("\\$v\\(" + NAME_ID + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {
				SqlVar v = null;
				try {
					try {
						v = vm.getVar(Integer.valueOf(m.group(1)));
					} catch (NumberFormatException e) {
						v = vm.getVar(m.group(1));
					}
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"字段"), e);
				}
				return sd.varPrefix() + v.toString();
			}

		};
	}

	public DefaultFormat(PatternFormat... pf) {
		super(pf);
	}

	public DefaultFormat(SqlDialect sd, Object obj) {
		this(get(obj, sd));
	}
}
